<?php
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Transforme une expression de calcul entrée par l'utilisateur/trice en expression interprétable en JS
 * @param string $expr
 * @return string
**/
function saisie_calcul_2_js($expr) {
	include_spip('inc/saisies');
	$expr = saisie_calcul_securiser($expr);
	preg_match_all('#@(.*)@#U', $expr, $matches);
	if (!$matches) {
		return;
	}
	$saisies_masquees = array_keys(saisies_lister_par_nom(saisies_afficher_si_liste_masquees('get')));//Retrouver les saisies masquées par afficher_si, car elle ne sont vidées qu'à la toute fin du formulaire (je ne sais plus pourquoi du reste, mais il y avait sans doute une très bonne raison)
	foreach ($matches[1] as $champ) {
		if (in_array($champ, $saisies_masquees)) {
			$expr = str_replace("@$champ@", 0, $expr);
		} elseif ($valeur = saisies_request($champ)) {
			if (is_array($valeur)) {
				$valeur = array_sum($valeur);
			}
			$expr = str_replace("@$champ@", $valeur, $expr);
		} else {
			$expr = str_replace("@$champ@", "get_valeur_saisie_pour_calcul(&quot;$champ&quot;)", $expr);
		}
	}
	$expr = str_replace("ROUND","calcul_arrondi",$expr);
	$expr = str_replace("\n",'',$expr);
	$expr = str_replace("\r",'',$expr);
	return $expr;
}


/**
 * Transforme une expression de calcul entrée par l'utilisateur/trice en expression interprétable en PHP
 * @param string $expr
 * @return string
**/
function saisie_calcul_2_php($expr) {
	$expr = saisie_calcul_securiser($expr);
	$expr = preg_replace("#@(.*)@#U", 'floatval(saisie_calcul_request(\'$1\'))', $expr);
	$expr = str_replace("ROUND","saisie_calcul_arrondi",$expr);
	return $expr;
}
/**
 * Récupère une valeur en _request
 * Si c'est un int, return directement
 * Si c'est un tableau, fait la somme
 * @param string $champ
 * @return int
**/
function saisie_calcul_request(string $champ): int{
	include_spip('inc/saisies');
	$valeur = saisies_request($champ);
	if (is_array($valeur)) {
		return array_sum($valeur);
	} else {
		return intval($valeur);
	}
}
/**
 * Une fonction d'arrondi php qui imite la fonction d'arrondi JS
 * @param int $valeur
 * @param int
**/
function saisie_calcul_arrondi($valeur, $precision = 0) {
	if ($valeur > 0) {
		return round($valeur, $precision, PHP_ROUND_HALF_UP);
	} else {
		return round($valeur, $precision, PHP_ROUND_HALF_DOWN);
	}
}

/**
 * Sécurise une expression de calcul passée en paramètre.
 * N'autorise que :
 * - texte entre @@
 * - opérateurs des opérations de base (+*-/)
 * - parenthèses
 * - point, virgule
 * - nombre
 * - ROUND (pour l'arrondi)
 * @param string $expr
 * @return string $expr soit l'expression, soit rien si jamais cela ne respect pas les règles
**/
function saisie_calcul_securiser($expr) {
	$hors_arobase = "#("
		."\d|"
		."\(|\)|"
		."\+|"
		."\*|"
		."\/|"
		."-|"
		."\.|"
		."\,|"
		."ROUND|"
		.")#";
	$arobase = "#@.*@#U";
	$valable = preg_replace($hors_arobase,'',$expr);
	$valable = preg_replace($arobase,'',$valable);
	if (!trim($valable)) {//Si à la fin il ne reste plus rien, c'est que c'est bon, on retourne donc l'expression
		return $expr;
	} else{//Sinon c'est le mal, et on retourne le vide
		return '';
	}
	return $expr;
}
